import os
from bootstrap import app

class Config:

    def __init__(self):
        return
    
    # To get the value of a key from the config file
    def get(self, key):
        # A sample of usage of the method
        # from laraflask.Helpers.config import config
        # config().get('app.app_name')
        # where app is the name of the file inside the config folder
        # app_name is the key

        # Split the key to get the file name and the key
        key = key.split('.')

        # Get the file name
        file_name = key[0]

        # Get the key
        key = key[1]

        # config file location
        config_location = app.AppBootstrap().app_config_path

        # config class name
        config_class_name = file_name.capitalize() + 'Configuration'

        # config file location
        config_file = config_location + '/' + file_name + '.py'

        # Check if the file exists
        if not os.path.isfile(config_file):
            return 'The file does not exist'
        
        # Import the file
        config_module = __import__('config.' + file_name, fromlist=[config_class_name])

        # Get the class
        config_class = getattr(config_module, config_class_name)

        # Get the value of the key
        return getattr(config_class(), key)




from flask import Blueprint, send_file

# Import bootstrap file
from bootstrap.app import AppBootstrap

import os

# Create a class for assets routes process
class AssetsRoute:
    def __init__(self):
        # Define assets mime type
        self.defined_assets_mime_type = {
            'css': 'text/css',
            'js': 'text/javascript',
            'jpg': 'image/jpeg',
            'jpeg': 'image/jpeg',
            'png': 'image/png',
            'gif': 'image/gif',
            'svg': 'image/svg+xml',
            'ico': 'image/x-icon',
            'webp': 'image/webp',
            'pdf': 'application/pdf',
            'mp4': 'video/mp4',
            'mp3': 'audio/mpeg',
            'wav': 'audio/wav',
            'ogg': 'audio/ogg',
            'scss': 'text/scss',
        }

        # default mime type
        self.default_mime_type = 'text/plain'

        return

    def register(self, path):
        # Get end of endpoint path
        # css/tes.css -> test.css
        # js/test.js -> test.js
        # img/test.jpg -> test.jpg
        filename = path.split('/')[-1]

        # Get the file extension
        file_extension = filename.split('.')[-1]

        # Get the assets path
        assets_path = AppBootstrap().app_static_path
        requested_file_path = assets_path + '/' + path

        # Check if the file exists
        if not os.path.isfile(requested_file_path):
            return 'The file '+ requested_file_path +' does not exist'

        # Get the requested file
        requested_file = open(requested_file_path, 'rb')

        # Check if the file extension is in the defined assets mime type
        if file_extension in self.defined_assets_mime_type:
            # Get the file mime type
            file_mime_type = self.defined_assets_mime_type[file_extension]
        else:
            file_mime_type = self.default_mime_type

        # Return the requested file
        return send_file(requested_file, mimetype=file_mime_type)
    
# Create a Blueprint for assets routes
assets_bp = Blueprint('assets', __name__)

@assets_bp.route('/<path:path>')
def assets(path):
    return AssetsRoute().register(path)



# Import app configuration class
from config.app import AppConfiguration

# Import app.py bootstrap file
from bootstrap.app import AppBootstrap

# Import service provider
from app.Providers.RouteServiceProvider import RouteServiceProvider

# Import middleware
from laraflask.Http.Middleware.ExcludeCSRF import ExcludeCSRF

# Import Flask
from flask import Flask

# Install necessary packages
from flask_wtf.csrf import CSRFProtect
from flask import request

# Create a Core class
class Core:

    # Initialize the Core class
    def __init__(self):
        # Create a Flask app
        self.app = Flask(__name__, 
                        template_folder=AppBootstrap().app_templates_path,
                        static_folder=AppBootstrap().app_static_path,
                        root_path=AppBootstrap().app_base_path
                    )
        
        # Set the app secret key
        self.app.secret_key = AppConfiguration().secret_key

        # Create a CSRF protection
        self.csrf = CSRFProtect()

    # Run the Flask app
    def run(self):
        # Register before request middleware
        @self.app.before_request 
        def exclude_csrf():
            # Register the ExcludeCSRF middleware
            return ExcludeCSRF(
                app=self.app,
                request=request,
                csrf=self.csrf
            ).register()
        
        # Register after request middleware
        @self.app.after_request
        def after_request(response):
            return response

        # Register routes
        self.register_routes()

        # Register CSRF protection
        self.register_csrf()

        # Run the Flask app
        return self.app.run(
            host=AppConfiguration().app_host,
            port=AppConfiguration().app_port,
            debug=AppConfiguration().app_debug
        )

    # Register routes
    def register_routes(self):
        return RouteServiceProvider(self.app).boot()
    
    # Register CSRF protection
    def register_csrf(self):
        return self.csrf.init_app(self.app)

    # Call the Flask app
    def __call__(self):
        return self.app